from labelbox.client import Client
from ._converter import _lb_converter

class coco_converter(_lb_converter):
    def __init__(self, project_id: str, client: Client, ontology_mapping: dict[str:str]) -> None:
        super().__init__(project_id, client, ontology_mapping)


import labelbox as lb
from .labelbox_converter import _lb_converter
from ultralytics.engine.results import Results
import numpy as np
from PIL import Image
import io

class Yolo_Converter(_lb_converter):
    def __init__(self, project_id: str, client: lb.Client, ontology_mapping: dict[str:str]) -> None:
        """Class to integrate YoloV8 annotations with labelbox

        Args:
            project_id (str): labelbox project id
            client (lb.client): labelbox client
            ontology_mapping (dict[str:str]): Dictionary mapping YoloV8 classes to labelbox feature name.
        """
        super().__init__(project_id, client, ontology_mapping)
    
    def create_yolo_bbox_annotation_predictions(yolo_results: Results, global_keys: list[str]) -> None:
        """Convert YOLOV8 model results to labelbox bbox annotations format

        Args:
            yolo_results (list[ultralytics.engine.results.Results]): YoloV8 prediction results.
            global_keys (list[str]): List of labelbox global keys for image. Must be same order as results.
        """
        
        results = []

        for i,result in enumerate(yolo_results):
            prediction = {"global_key": global_keys[i],
                          "predictions": []}
            for bbox in result.boxes:
                start_x, start_y, end_x, end_y = bbox.xyxy.tolist()[0]
                prediction["predictions"].append({"answer": bbox.cls, "points": {"start_x": start_x, "start_y": start_y, "end_x": end_x, "end_y": end_y}})
            
            results.append(prediction)

        super().create_bbox_labels(results)
    
    def create_yolo_segment_annotation_predictions(yolo_results: Results, global_keys: list[str], image_sizes: list[tuple[int, int]]) -> None:
        """Convert YOLOV8 model results to labelbox segment annotations format

        Args:
            yolo_results (list[ultralytics.engine.results.Results]): YoloV8 prediction results.
            global_keys (list[str]): List of labelbox global keys for image. Must be same order as results.
            image_sizes (list[tuple[int, int]]): List of image size (W, H). Must be same order as results
        """
        
        results = []

        for x,result in enumerate(yolo_results):
            prediction = {"global_key": global_keys[x],
                          "predictions": []}
            for i, mask in enumerate(result.masks.data):
                mask: np = (mask.numpy() * 255).astype("uint8")
                img = Image.fromarray(mask, "L")
                img = img.resize(image_sizes[x])
                img_byte_arr = io.BytesIO()
                img.save(img_byte_arr, format='PNG')
                encoded_image_bytes = img_byte_arr.getvalue()
                prediction["predictions"].append({
                                                "answer": int(result.boxes[i].cls), 
                                                "mask": encoded_image_bytes,
                                                "color": (255,255,255)
                                                })
            results.append(prediction)


        super().create_segment_labels(results)
    
    def create_yolo_polygon_annotation_predictions(yolo_results: Results, global_keys: list[str]) -> None:
        """Convert YOLOV8 model results to labelbox polygon annotations format

        Args:
            yolo_results (list[ultralytics.engine.results.Results]): YoloV8 prediction results.
            global_keys (list[str]): List of labelbox global keys for image. Must be same order as results.
        """
        
        results = []

        for x,result in enumerate(yolo_results):
            prediction = {"global_key": global_keys[x],
                          "predictions": []}
            for i, coordinates in enumerate(result.masks.xy):
                prediction["predictions"].append({"answer": int(result.boxes[i].cls), "points": [(coordinate[0],coordinate[1]) for coordinate in coordinates]})
            
            results.append(prediction)
        
        super().create_polygon_labels(results)


import labelbox as lb
import labelbox.types as lb_types
from uuid import uuid4

class _lb_converter:
    def __init__(self, project_id: str, client: lb.Client, ontology_mapping: dict[str: str]) -> None:
        self.client: lb.Client = client
        self.project: str = project_id
        self.ontology_mapping: dict[str:str] = ontology_mapping
        self.labels = []
        
    def export_data_row(self) -> list[dict[str:str]]:
        project = self.client.get_project(self.project)
        
        self.client.enable_experimental = True

        export_params= {
        "attachments": False,
        "metadata_fields": False,
        "data_row_details": True,
        "project_details": True,
        "label_details": True,
        "performance_details": False,
        "interpolated_frames": True
        }

        export_task = project.export(params=export_params)
        export_task.wait_till_done()
        
        data_rows = []

        if export_task.has_errors():
            export_task.get_stream(
            converter=lb.JsonConverter(),
            stream_type=lb.StreamType.ERRORS
            ).start(stream_handler=lambda error: print(error))

        if export_task.has_result():
            stream = export_task.get_stream(
            converter=lb.JsonConverter(),
            stream_type=lb.StreamType.RESULT
            )
            for output in stream:
                data_rows.append(output.json_str)
        
        return data_rows
    
    def import_ground_truths(self, mal_job_name = f"mal-{str(uuid4())}"):
        if len(self.labels) == 0:
            raise Exception("No labels to import")
        upload_job = lb.LabelImport.create_from_objects(
                        client = self.client,
                        project_id = self.project,
                        name=mal_job_name,
                        labels=self.labels
                        )
        upload_job.wait_until_done()
        
        return upload_job
    
    def import_predictions(self, mal_job_name = f"mal-{str(uuid4())}"):
        if len(self.labels) == 0:
            raise Exception("No labels to import")
        upload_job = lb.MALPredictionImport.create_from_objects(
                        client = self.client,
                        project_id = self.project,
                        name=mal_job_name,
                        predictions=self.labels
                        )
        upload_job.wait_until_done()
        
        return upload_job


    def create_bbox_labels(self, results: list[dict[str:str]] ):
        """Creates bbox labels that can then be imported

        Args:
            results (list[dict[str:str]]): [{
                                            global_key: <Global_Key>, 
                                            predictions: [{
                                                           answer: <model_answer_name>, 
                                                           points: {start_x, start_y, end_x, end_y}
                                                         }]
                                            }]
        """
        for result in results:
            annotations = []
            if "predictions" in result:
                for prediction in result["predictions"]:
                    lb_name = self.ontology_mapping[prediction["answer"]]
                    bbox_source = lb_types.ObjectAnnotation(
                        name=lb_name,
                        value=lb_types.Rectangle(
                            start=lb_types.Point(x=prediction["start_x"], y=prediction["start_y"]),
                            end=lb_types.Point(x=prediction["end_x"], y=prediction["end_y"])
                        ))
                    annotations.append(bbox_source)
                self.labels.append(
                    lb_types.Label(data=lb_types.ImageData(global_key=result["global_key"]), annotations=annotations)
                    )
    def create_segment_labels(self, results: list[dict[str:str]] ):
        """Creates segment labels that can then be imported

        Args:
            results (list[dict[str:str]]): [{
                                            global_key: <Global_Key>, 
                                            predictions: [{
                                                           answer: <model_answer_name>, 
                                                           mask: <byte_array>,
                                                           color(optional): <rgb tuple> 
                                                         }]
                                           }]
        """
        for result in results:
            annotations = []
            if "predictions" in result:
                for prediction in result["predictions"]:
                    color = (255,255,255)
                    if "color" in prediction:
                        color = prediction["color"]
                    lb_name = self.ontology_mapping[prediction["answer"]]
                    mask_data = lb_types.MaskData(im_bytes=prediction["mask"])
                    mask_annotation = lb_types.ObjectAnnotation(
                        name=lb_name,
                        value=lb_types.Mask(
                            mask=mask_data,
                            color=color
                        ))
                    annotations.append(mask_annotation)
                self.labels.append(
                    lb_types.Label(data=lb_types.ImageData(global_key=result["global_key"]), annotations=annotations)
                    )
    def create_polygon_labels(self, results: list[dict[str:str]] ):
        """Creates polygon labels that can then be imported

        Args:
            results (list[dict[str:str]]): [{
                                            global_key: <Global_Key>, 
                                            predictions: [{
                                                           answer: <model_answer_name>, 
                                                           points: [(<x_coordinate>, <y_coordinate>)]
                                                         }]
                                           }]
        """
        for result in results:
            annotations = []
            if "predictions" in result:
                for prediction in result["predictions"]:
                    lb_name = self.ontology_mapping[prediction["answer"]]
                    polygon_annotation = lb_types.ObjectAnnotation(
                        name=lb_name,
                        value=lb_types.Polygon(
                            points=[lb_types.Point(x=coordinates[0], y=coordinates[1]) for coordinates in prediction["points"]]
                        ))
                    annotations.append(polygon_annotation)
                self.labels.append(
                    lb_types.Label(data=lb_types.ImageData(global_key=result["global_key"]), annotations=annotations)
                    )
        
